﻿namespace TestProject
{
    /// <summary>
    /// WAVE FMT
    /// </summary>
    public class WAVEFMT
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// 청크 ID
        /// </summary>
        /// <remarks>
        /// 4바이트 : 빅 엔디안
        /// 'fmt '라는 문자를 저장한다.
        /// </remarks>
        public string ChunkID;

        /// <summary>
        /// 청크 크기
        /// </summary>
        /// <remarks>
        /// 4바이트 : 리틀 엔디안
        /// ChunkSize 항목 이후 항목들의 크기를 저장한다.
        /// 고정 값 16을 저장한다.
        /// </remarks>
        public uint ChunkSize;

        /// <summary>
        /// 오디오 포맷
        /// </summary>
        /// <remarks>
        /// 2바이트 : 리틀 엔디안
        /// PCM의 경우 1을 저장한다.
        /// </remarks>
        public ushort AudioFormat;

        /// <summary>
        /// 채널 수
        /// </summary>
        /// <remarks>
        /// 2바이트 : 리틀 엔디안
        /// 모노     : 1, [data][data][data]...
        /// 스테레오 : 2, [left][right]...
        /// 3 채널   : 3, [left][right][center]...
        /// 쿼드     : 4, [front left][front right][rear left][rear right]...
        /// 4 채널   : 5, [left][center][right][surround]...
        /// 6 채널   : 6, [left center][left][center][right center][right][surround]...
        /// </remarks>
        public ushort ChannelCount;

        /// <summary>
        /// 초당 샘플 수
        /// </summary>
        /// <remarks>
        /// 4바이트 : 리틀 엔디안
        /// </remarks>
        public uint SampleCountPerSecond;

        /// <summary>
        /// 초당 평균 바이트 수
        /// </summary>
        /// <remarks>
        /// 4바이트 : 리틀 엔디안
        /// </remarks>
        public uint AverageByteCountPerSecond;

        /// <summary>
        /// 블럭 정렬
        /// </summary>
        /// <remarks>
        /// 2바이트 : 리틀 엔디안
        /// 샘플 프레임 크기
        /// 모노   : 샘플 크기 * 1
        /// 스테렝 : 샘플 크기 * 2
        /// </remarks>
        public ushort BlockAlignment;

        /// <summary>
        /// 샘플당 비트 수
        /// </summary>
        /// <remarks>
        /// 2바이트 : 리틀 엔디안
        /// </remarks>
        public ushort BitCountPerSample;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - WAVEFMT()

        /// <summary>
        /// 생성자
        /// </summary>
        public WAVEFMT()
        {
            ChunkID                   = "fmt ";
            ChunkSize                 = 16;
            AudioFormat               = 1;
            ChannelCount              = 2;
            SampleCountPerSecond      = 44100;
            BitCountPerSample         = 16;
            BlockAlignment            = (ushort)(ChannelCount * (BitCountPerSample / 8));
            AverageByteCountPerSecond = SampleCountPerSecond * BlockAlignment;            
        }

        #endregion
    }
}