﻿using System;
using System.IO;

namespace TestProject
{
    /// <summary>
    /// WAVE 제너레이터
    /// </summary>
    public class WaveGenerator
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 헤더
        /// </summary>
        private WAVERIFF header;

        /// <summary>
        /// 포맷
        /// </summary>
        private WAVEFMT format;

        /// <summary>
        /// 데이터
        /// </summary>
        private WAVEDATA data;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - WaveGenerator()

        /// <summary>
        /// 생성자
        /// </summary>
        public WaveGenerator()
        {
            this.header = new WAVERIFF();
            this.format = new WAVEFMT();
            this.data   = new WAVEDATA();

            uint sampleCount = this.format.SampleCountPerSecond * this.format.ChannelCount;

            this.data.ValueArray = new short[sampleCount];

            int    amplitude = 32760;  // 16비트 오디오용 최대 진폭
            double frequency = 440.0d; // Concert A : 440Hz
            double tone      = (Math.PI * 2 * frequency) / sampleCount;

            for(uint i = 0; i < sampleCount - 1; i++)
            {
                for(int channel = 0; channel < this.format.ChannelCount; channel++)
                {
                    this.data.ValueArray[i + channel] = Convert.ToInt16(amplitude * Math.Sin(tone * i));
                }
            }

            this.data.ChunkSize = (uint)(this.data.ValueArray.Length * (this.format.BitCountPerSample / 8));
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 저장하기 - Save(filePath)

        /// <summary>
        /// 저장하기
        /// </summary>
        /// <param name="filePath">파일 경로</param>
        public void Save(string filePath)
        {
            FileStream   stream = new FileStream(filePath, FileMode.Create);
            BinaryWriter writer = new BinaryWriter(stream);

            // RIFF
            writer.Write(header.ChunkID.ToCharArray());
            writer.Write(header.ChunkSize);
            writer.Write(header.Format.ToCharArray());

            // FMT
            writer.Write(format.ChunkID.ToCharArray());
            writer.Write(format.ChunkSize);
            writer.Write(format.AudioFormat);
            writer.Write(format.ChannelCount);
            writer.Write(format.SampleCountPerSecond);
            writer.Write(format.AverageByteCountPerSecond);
            writer.Write(format.BlockAlignment);
            writer.Write(format.BitCountPerSample);

            // DATA
            writer.Write(data.ChunkID.ToCharArray());
            writer.Write(data.ChunkSize);

            foreach(short value in data.ValueArray)
            {
                writer.Write(value);
            }

            writer.Seek(4, SeekOrigin.Begin);

            uint filesize = (uint)writer.BaseStream.Length;

            writer.Write(filesize - 8);

            writer.Close();

            stream.Close();
        }

        #endregion
    }
}